@@ -108,8 +108,7 @@ setReplaceMethod("colnames", "DataFrame",
 
 .valid.DataFrame.names <- function(x)
 {
-  ## DataFrames with no columns can have NULL column name
-  if (is.null(names(x)) && ncol(x) != 0)
+  if (is.null(names(x)))
     return("column names should not be NULL")
   if (length(names(x)) != ncol(x))
     return("number of columns and number of column names differ")
@@ -182,7 +181,7 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE)
     if (!all(nrows == nr))
       stop("different row counts implied by arguments")
     varlist <- unlist(varlist, recursive = FALSE, use.names = FALSE)
-    nms <- unlist(varnames[ncols > 0L])
+    nms <- as.character(unlist(varnames[ncols > 0L]))
     if (check.names)
       nms <- make.names(nms, unique = TRUE)
     names(varlist) <- nms
@@ -236,7 +235,8 @@ setReplaceMethod("[[", "DataFrame",
 setMethod("extractROWS", "DataFrame",
     function(x, i)
     {
-        i <- normalizeSingleBracketSubscript(i, x, exact=FALSE, as.NSBS=TRUE)
+        i <- normalizeSingleBracketSubscript(i, x, exact=FALSE, allow.NAs=TRUE,
+                                                   as.NSBS=TRUE)
         slot(x, "listData", check=FALSE) <-
             lapply(structure(seq_len(ncol(x)), names=names(x)),
                    function(j) extractROWS(x[[j]], i))
@@ -560,6 +560,14 @@ setAs("data.frame", "DataFrame",
         new2("DataFrame", listData=from, nrows=nr, rownames=rn, check=FALSE)
       })
 
+setAs("data.table", "DataFrame",
+    function(from)
+    {
+        df <- data.table:::as.data.frame.data.table(from)
+        as(df, "DataFrame")
+    }
+)
+
 setAs("table", "DataFrame",
       function(from) {
         df <- as.data.frame(from)
