@@ -53,6 +53,9 @@ setReplaceMethod("active", "FilterRules", function(x, value) {
   } else stop("unsupported type of 'value'")
 })
 
+setMethod("parallelSlotNames", "FilterRules",
+          function(x) c("active", callNextMethod()))
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Constructor.
 ###
@@ -73,7 +76,7 @@ FilterRules.parseRule <- function(expr) {
 ## takes logical expressions, character vectors, or functions to parse
 
 FilterRules <- function(exprs = list(), ..., active = TRUE) {
-  exprs <- c(as.list(substitute(list(...)))[-1L], exprs)
+  exprs <- c(exprs, as.list(substitute(list(...)))[-1L])
   if (length(names(exprs)) == 0) {
     funs <- as.logical(sapply(exprs, is.function))
     nonfuns <- exprs[!funs]
@@ -123,18 +126,6 @@ setReplaceMethod("[[", "FilterRules",
                    x
                  })
 
-setMethod("[", "FilterRules",
-          function(x, i, j, ..., drop)
-          {
-            if (!missing(j) || length(list(...)) > 0)
-              stop("invalid subsetting")
-            if (!missing(i)) {
-              x@active <- setNames(setNames(x@active, names(x))[i], NULL)
-              x <- callNextMethod(x, i)
-            }
-            x
-          })
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Validity.
 ###
@@ -238,6 +229,9 @@ setMethod("eval", signature(expr="FilterRules", envir="ANY"),
                     min(NROW(envir), length(val)) != 0)))
                 stop("filter rule evaluated to inconsistent length: ",
                      names(rule)[i])
+              if (anyNA(val)) {
+                  val[is.na(val)] <- FALSE
+              }
               if (length(rules) > 1L)
                 envir <- extractROWS(envir, val)
               result[result] <- val
