@@ -419,8 +419,8 @@ combine_Hits_objects <- function(Class, objects,
     ## TODO: Implement (in C) fast 'elementIs(objects, class)' in S4Vectors
     ## that does 'sapply(objects, is, class, USE.NAMES=FALSE)', and use it
     ## here. 'elementIs(objects, "NULL")' should work and be equivalent to
-    ## 'elementIsNull(objects)'.
-    if (!all(sapply(objects, is, Class, USE.NAMES=FALSE)))
+    ## 'sapply_isNULL(objects)'.
+    if (!all(vapply(objects, is, logical(1), Class, USE.NAMES=FALSE)))
         stop("the objects to combine must be ", Class, " objects (or NULLs)")
     objects_names <- names(objects)
     names(objects) <- NULL  # so lapply(objects, ...) below returns an
@@ -453,7 +453,7 @@ combine_Hits_objects <- function(Class, objects,
     if (ignore.mcols) {
         ans_mcols <- NULL
     } else {
-        ans_mcols <- do.call(S4Vectors:::rbind_mcols, objects)
+        ans_mcols <- do.call(rbind_mcols, objects)
     }
 
     ## Make 'ans' and return it.
@@ -495,6 +495,22 @@ selectHits <- function(x, select=c("all", "first", "last", "arbitrary",
                           PACKAGE="S4Vectors")
 }
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### breakTies()
+###
+### Redundant with selectHits. The only difference is that it returns a Hits
+### object. That alone doesn't justify introducing a new verb. Should be
+### controlled via an extra arg to selectHits() e.g. 'as.Hits' (FALSE by
+### default). H.P. -- Oct 16, 2016
+
+breakTies <- function(x, method=c("first", "last")) {
+    if (!is(x, "Hits"))
+        stop("'x' must be a Hits object")
+    method <- match.arg(method)
+    to <- selectHits(x, method)
+    .new_Hits("SortedByQueryHits", which(!is.na(to)), to[!is.na(to)],
+              nLnode(x), nRnode(x), NULL)
+}
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### revmap()
@@ -515,6 +531,7 @@ revmap_Hits <- function(x)
     new_Hits(class(x), to(x), from(x), nRnode(x), nLnode(x), mcols(x))
 
 ### FIXME: Replace this with "revmap" method for Hits objects.
+t.Hits <- function(x) t(x)
 setMethod("t", "Hits", revmap_Hits)
 
 
