@@ -55,13 +55,7 @@ setMethod("elementNROWS", "List",
 ### clearly a misnomer). For backward compatibility the old name was kept
 ### around, deprecated, and became an "alias" for elementNROWS().
 setGeneric("elementLengths", function(x) standardGeneric("elementLengths"))
-setMethod("elementLengths", "ANY",
-    function(x)
-    {
-        .Deprecated("elementNROWS")
-        elementNROWS(x)
-    }
-)
+setMethod("elementLengths", "ANY", function(x) .Defunct("elementNROWS"))
 
 setGeneric("isEmpty", function(x) standardGeneric("isEmpty"))
 setMethod("isEmpty", "ANY",
@@ -143,7 +137,7 @@ setMethod("show", "List",
 ### unlist()
 ###
 
-### NOT exported. Assumes 'names1' is not NULL.
+### NOT exported. Assume 'names1' is not NULL.
 make_unlist_result_names <- function(names1, names2)
 {
     if (is.null(names2))
@@ -162,9 +156,6 @@ make_unlist_result_names <- function(names1, names2)
 setMethod("unlist", "List",
     function(x, recursive=TRUE, use.names=TRUE)
     {
-        if (!identical(recursive, TRUE))
-            stop("\"unlist\" method for List objects ",
-                 "does not support the 'recursive' argument")
         if (!isTRUEorFALSE(use.names))
             stop("'use.names' must be TRUE or FALSE")
         if (length(x) == 0L)
@@ -206,7 +197,7 @@ setMethod("unlist", "List",
 ### Subsetting.
 ###
 
-### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+### Assume 'x' and 'i' are parallel List objects (i.e. same length).
 ### Returns TRUE iff 'i' contains non-NA positive values that are compatible
 ### with the shape of 'x'.
 .is_valid_NL_subscript <- function(i, x)
@@ -223,7 +214,7 @@ setMethod("unlist", "List",
     return(TRUE)
 }
 
-### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+### Assume 'x' and 'i' are parallel List objects (i.e. same length).
 ### Returns the name of one of the 3 supported fast paths ("LL", "NL", "RL")
 ### or NA if no fast path can be used.
 .select_fast_path <- function(i, x)
@@ -261,7 +252,7 @@ setMethod("unlist", "List",
     return(NA_character_)
 }
 
-### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+### Assume 'x' and 'i' are parallel List objects (i.e. same length).
 ### Truncate or recycle each list element of 'i' to the length of the
 ### corresponding element in 'x'.
 .adjust_elt_lengths <- function(i, x)
@@ -277,7 +268,7 @@ setMethod("unlist", "List",
     return(i)
 }
 
-### Assumes 'x' and 'i' are parallel List objects (i.e. same length),
+### Assume 'x' and 'i' are parallel List objects (i.e. same length),
 ### and 'i' is a List of logical vectors or logical-Rle objects.
 .unlist_LL_subscript <- function(i, x)
 {
@@ -285,7 +276,7 @@ setMethod("unlist", "List",
     unlist(i, use.names=FALSE)
 }
 
-### Assumes 'x' and 'i' are parallel List objects (i.e. same length),
+### Assume 'x' and 'i' are parallel List objects (i.e. same length),
 ### and 'i' is a List of numeric vectors or numeric-Rle objects.
 .unlist_NL_subscript <- function(i, x)
 {
@@ -294,7 +285,7 @@ setMethod("unlist", "List",
     unlist(i, use.names=FALSE)
 }
 
-### Assumes 'x' and 'i' are parallel List objects (i.e. same length),
+### Assume 'x' and 'i' are parallel List objects (i.e. same length),
 ### and 'i' is a List of Ranges objects.
 .unlist_RL_subscript <- function(i, x)
 {
@@ -304,7 +295,9 @@ setMethod("unlist", "List",
 }
 
 ### Fast subset by List of logical vectors or logical-Rle objects.
-### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+### Assume 'x' and 'i' are parallel List objects (i.e. same length).
+### Propagate 'names(x)' only. Caller is responsible for propagating 'mcols(x)'
+### and 'metadata(x)'.
 .fast_subset_List_by_LL <- function(x, i)
 {
     ## Unlist 'x' and 'i'.
@@ -317,14 +310,15 @@ setMethod("unlist", "List",
     ## Relist.
     group <- rep.int(seq_along(x), elementNROWS(x))
     group <- extractROWS(group, unlisted_i)
-    ans_skeleton <- IRanges::PartitioningByEnd(group, NG=length(x), names=names(x))
-    ans <- as(relist(unlisted_ans, ans_skeleton), class(x))
-    metadata(ans) <- metadata(x)
-    ans
+    ans_skeleton <- IRanges::PartitioningByEnd(group, NG=length(x),
+                                               names=names(x))
+    relist(unlisted_ans, ans_skeleton)
 }
 
 ### Fast subset by List of numeric vectors or numeric-Rle objects.
-### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+### Assume 'x' and 'i' are parallel List objects (i.e. same length).
+### Propagate 'names(x)' only. Caller is responsible for propagating 'mcols(x)'
+### and 'metadata(x)'.
 .fast_subset_List_by_NL <- function(x, i)
 {
     ## Unlist 'x' and 'i'.
@@ -337,13 +331,13 @@ setMethod("unlist", "List",
     ## Relist.
     ans_breakpoints <- cumsum(unname(elementNROWS(i)))
     ans_skeleton <- IRanges::PartitioningByEnd(ans_breakpoints, names=names(x))
-    ans <- as(relist(unlisted_ans, ans_skeleton), class(x))
-    metadata(ans) <- metadata(x)
-    ans
+    relist(unlisted_ans, ans_skeleton)
 }
 
 ### Fast subset by List of Ranges objects.
-### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+### Assume 'x' and 'i' are parallel List objects (i.e. same length).
+### Propagate 'names(x)' only. Caller is responsible for propagating 'mcols(x)'
+### and 'metadata(x)'.
 .fast_subset_List_by_RL <- function(x, i)
 {
     ## Unlist 'x' and 'i'.
@@ -356,9 +350,7 @@ setMethod("unlist", "List",
     ## Relist.
     ans_breakpoints <- cumsum(unlist(sum(width(i)), use.names=FALSE))
     ans_skeleton <- IRanges::PartitioningByEnd(ans_breakpoints, names=names(x))
-    ans <- as(relist(unlisted_ans, ans_skeleton), class(x))
-    metadata(ans) <- metadata(x)
-    ans
+    relist(unlisted_ans, ans_skeleton)
 }
 
 ### Subset a List object by a list-like subscript.
@@ -407,7 +399,11 @@ subset_List_by_List <- function(x, i)
                                     LL=.fast_subset_List_by_LL,
                                     NL=.fast_subset_List_by_NL,
                                     RL=.fast_subset_List_by_RL)
-            return(fast_path_FUN(x, i))  # fast path
+            ans <- as(fast_path_FUN(x, i), class(x))  # fast path
+            ## Propagate 'metadata(x)' and 'mcols(x)'.
+            metadata(ans) <- metadata(x)
+            mcols(ans) <- mcols(x)
+            return(ans)
         }
     }
     ## Slow path (loops over the list elements of 'x').
@@ -427,7 +423,7 @@ subset_List_by_List <- function(x, i)
     rep(value, length.out=i_len)
 }
 
-### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+### Assume 'x' and 'i' are parallel List objects (i.e. same length).
 .fast_lsubset_List_by_List <- function(x, i, value)
 {
     ## Unlist 'x', 'i', and 'value'.
@@ -628,24 +624,8 @@ setAs("List", "list", function(from) as.list(from))
 }
 setMethod("as.list", "List", .as.list.List)
 
-setMethod("as.env", "List",
-          function(x, enclos = parent.frame(2), tform = identity) {
-              nms <- names(x)
-              if (is.null(nms))
-                  stop("cannot convert to environment when names are NULL")
-              env <- new.env(parent = enclos)
-              lapply(nms,
-                     function(col) {
-                         colFun <- function() {
-                             val <- tform(x[[col]])
-                             rm(list=col, envir=env)
-                             assign(col, val, env)
-                             val
-                         }
-                         makeActiveBinding(col, colFun, env)
-                     })
-              env
-          })
+setMethod("parallelVectorNames", "List",
+          function(x) setdiff(callNextMethod(), c("group", "group_name")))
 
 listClassName <- function(impl, element.type) {
   if (is.null(impl))
