@@ -214,9 +214,9 @@ setMethod("within", "List",
           function(data, expr, ...)
           {
             ## cannot use active bindings here, as they break for replacement
-            e <- list2env(as.list(data))
-            ##e <- as.env(data)
-            safeEval(substitute(expr), e, top_prenv(expr))
+            enclos <- top_prenv(expr)
+            e <- list2env(as.list(data), parent=enclos)
+            safeEval(substitute(expr), e, enclos)
             l <- mget(ls(e), e)
             l <- l[!sapply(l, is.null)]
             nD <- length(del <- setdiff(names(data), (nl <- names(l))))
@@ -247,3 +247,49 @@ droplevels.List <- function(x, ...) droplevels(x, ...)
 }
 
 setMethod("droplevels", "List", .droplevels.List)
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Summarizing.
+###
+
+setMethod("anyNA", "List", function(x, recursive=FALSE) {
+    stopifnot(isTRUEorFALSE(recursive))
+    if (recursive) {
+        anyNA(as.list(x), recursive=TRUE)
+    } else {
+        callNextMethod()
+    }
+})
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Matrix construction
+###
+
+normBindArgs <- function(..., deparse.level=1L) {
+    stopifnot(isSingleNumber(deparse.level),
+              deparse.level >= 0L,
+              deparse.level <= 2L)
+    args <- list(...)
+    if (deparse.level > 0L) {
+        exprs <- as.list(substitute(list(...)))[-1L]
+        genName <- if (is.null(names(args))) TRUE else names(args) == ""
+        if (deparse.level == 1L && any(genName))
+            genName <- genName & vapply(exprs, is.name, logical(1L))
+        if (any(genName)) {
+            if (is.null(names(args)))
+                names(args) <- rep("", length(args))
+            names(args)[genName] <- as.character(exprs[genName])
+        }
+    }
+    args
+}
+
+setMethod("rbind", "List", function(..., deparse.level=1L) {
+    args <- normBindArgs(..., deparse.level=deparse.level)
+    do.call(rbind, lapply(args, as.list))
+})
+
+setMethod("cbind", "List", function(..., deparse.level=1L) {
+    args <- normBindArgs(..., deparse.level=deparse.level)
+    do.call(cbind, lapply(args, as.list))
+})
