@@ -107,7 +107,7 @@ setGeneric("values", function(x, ...) standardGeneric("values"))
 
 setMethod("values", "Vector", function(x, ...) elementMetadata(x, ...))
 
-setMethod("anyNA", "Vector", function(x) any(is.na(x)))
+setMethod("anyNA", "Vector", function(x, recursive=FALSE) any(is.na(x)))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -264,8 +264,12 @@ setMethod("as.matrix", "Vector", function(x) {
 
 makeFixedColumnEnv <- function(x, parent, tform = identity) {
   env <- new.env(parent=parent)
-  pvnEnv <- environment(selectMethod("parallelVectorNames", class(x)))
-  lapply(parallelVectorNames(x), function(nm) {
+  pkg <- packageSlot(class(x))
+  pvnEnv <- .GlobalEnv
+  if (!is.null(pkg)) {
+      pvnEnv <- getNamespace(pkg)
+  }
+  lapply(c("names", parallelVectorNames(x)), function(nm) {
     accessor <- get(nm, pvnEnv, mode="function")
     makeActiveBinding(nm, function() {
       val <- tform(accessor(x))
@@ -424,21 +428,33 @@ setReplaceMethod("[", "Vector",
     }
 )
 
-### Works on any Vector object for which c() and [ work. Assumes 'value' is
-### compatible with 'x'.
+### Work on any Vector object for which c() and extractROWS() work.
+### Assume 'value' is compatible with 'x'.
 setMethod("replaceROWS", "Vector",
     function(x, i, value)
     {
-        idx <- seq_along(x)
-        i <- extractROWS(setNames(idx, names(x)), i)
-        ## Assuming that c() works on objects of class 'class(x)' and does the
-        ## right thing (i.e. returns an object of the same class as 'x' and of
-        ## length 'length(x) + length(value)').
+        i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
+
+        ## --<1>-- Concatenate 'x' and 'value' with c() -----
+
+        ## We assume that c() works on objects of class 'class(x)' and that it
+        ## does the right thing i.e. that it returns an object of the same
+        ## class as 'x' and of length 'length(x) + length(value)'.
         ans <- c(x, value)
-        ## Assuming that [ works on objects of class 'class(x)'.
-        idx[i] <- length(x) + seq_along(value)
-        ans <- ans[idx]
-        ## Restore the original decoration.
+
+        ## --<2>-- Subset 'c(x, value)' with extractROWS() -----
+
+        idx <- replaceROWS(seq_along(x), i, seq_along(value) + length(x))
+        ## Because of how we constructed it, 'idx' is guaranteed to be a valid
+        ## subscript to use in 'extractROWS(ans, idx)'. By wrapping it inside a
+        ## NativeNSBS object, extractROWS() won't waste time checking it or
+        ## trying to normalize it.
+        idx <- NativeNSBS(idx, length(ans), TRUE, FALSE)
+        ## We assume that extractROWS() works on an object of class 'class(x)'.
+        ans <- extractROWS(ans, idx)
+
+        ## --<3>-- Restore the original decoration -----
+
         metadata(ans) <- metadata(x)
         names(ans) <- names(x)
         ## However, we want the replaced elements in 'x' to get their
@@ -446,6 +462,7 @@ setMethod("replaceROWS", "Vector",
         ## metadata columns. See this thread on bioc-devel:
         ##  https://stat.ethz.ch/pipermail/bioc-devel/2015-November/008319.html
         #mcols(ans) <- mcols(x)
+
         ans
     }
 )
@@ -521,28 +538,51 @@ setMethod("classNameForDisplay", "ANY",
    }
 )
 
-setMethod("classNameForDisplay", "AsIs", function(x) {
-  class(x) <- setdiff(class(x), "AsIs")
-  classNameForDisplay(x)
-})
+.drop_AsIs <- function(x)
+{
+    #x_class <- class(x)
+    #if (x_class[[1L]] == "AsIs")
+    #    class(x) <- x_class[-1L]
+
+    ## Simpler, and probably more robust, than the above.
+    class(x) <- setdiff(class(x), "AsIs")
+    x
+}
+
+setMethod("classNameForDisplay", "AsIs",
+    function(x) classNameForDisplay(.drop_AsIs(x))
+)
 
 
+### All "showAsCell" methods should return a character vector.
 setGeneric("showAsCell",
     function(object) standardGeneric("showAsCell")
 )
 
+setMethod("showAsCell", "AsIs",
+    function(object) showAsCell(.drop_AsIs(object))
+)
+
 setMethod("showAsCell", "ANY", function(object) {
+  if (NROW(object) == 0L)
+    return(character(0L))
   if (length(dim(object)) > 2)
     dim(object) <- c(nrow(object), prod(tail(dim(object), -1)))
   if (NCOL(object) > 1) {
-    class(object) <- setdiff(class(object), "AsIs")
     df <- as.data.frame(object[, head(seq_len(ncol(object)), 3), drop = FALSE])
-    attempt <- do.call(paste, df)
-    if (ncol(object) > 3)
-      attempt <- paste(attempt, "...")
+    attempt <- do.call(paste, c(df, sep=":"))
+    if (ncol(object) > 3L)
+      attempt <- paste0(attempt, ":...")
     attempt
   } else if (NCOL(object) == 0L) {
     rep.int("", NROW(object))
+  } else if (is.list(object) || is(object, "List")) {
+    vapply(object, function(x) {
+      str <- paste(head(x, 3L), collapse = ",") 
+      if (length(x) > 3L)
+        str <- paste0(str, ",...")
+      str
+    }, character(1L))
   } else {
     attempt <- try(as.vector(object), silent=TRUE)
     if (is(attempt, "try-error"))
@@ -551,9 +591,7 @@ setMethod("showAsCell", "ANY", function(object) {
   }
 })
 
-setMethod("showAsCell", "Vector", function(object)
-          rep.int("########", length(object)))
-
+### Mmmh... these methods don't return a character vector. Is that ok?
 setMethod("showAsCell", "Date", function(object) object)
 setMethod("showAsCell", "POSIXt", function(object) object)
 
