@@ -60,7 +60,7 @@ setMethods("<", .OP2_SIGNATURES, function(e1, e2) { !(e2 <= e1) })
 
 setMethods(">", .OP2_SIGNATURES, function(e1, e2) { !(e1 <= e2) })
 
-compare <- function(...) {.Deprecated("pcompare"); pcompare(...)}
+compare <- function(...) .Defunct("pcompare")
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -74,6 +74,16 @@ setGeneric("selfmatch", function(x, ...) standardGeneric("selfmatch"))
 ### Default "selfmatch" method. Args in ... are propagated to match().
 setMethod("selfmatch", "ANY", function(x, ...) match(x, x, ...))
 
+### Optimized "selfmatch" method for factors.
+setMethod("selfmatch", "factor", function(x, ..., incomparables = NULL) {
+    ignore.na <- isTRUE(is.na(incomparables))
+    has.incomparables <- !is.null(incomparables) && !ignore.na
+    if (!missing(...) || has.incomparables || (!ignore.na && anyNA(x)) ||
+          is.unsorted(x))
+        callNextMethod()
+    else as.integer(x)
+})
+
 ### 'selfmatch_mapping' must be an integer vector like one returned by
 ### selfmatch(), that is, values are non-NAs and such that any value 'v' in it
 ### must appear for the first time at *position* 'v'.
@@ -279,10 +289,17 @@ sort.Vector <- function(x, decreasing=FALSE, ...)
     sort(x, decreasing=decreasing, ...)
 setMethod("sort", "Vector", .sort.Vector)
 
-formulaAsListCall <- function(formula) attr(terms(formula), "variables")
+formulaAsListCall <- function(formula) {
+    attr(terms(formula), "variables")
+}
 
 formulaValues <- function(x, formula) {
-    eval(formulaAsListCall(formula), as.env(x, environment(formula)))
+    listCall <- formulaAsListCall(formula)
+    vals <- eval(listCall, as.env(x, environment(formula)))
+    names(vals) <- vapply(listCall, function(x) {
+        paste(deparse(x, width.cutoff = 500), collapse = " ")
+    }, character(1L))[-1L]
+    vals
 }
 
 orderBy <- function(formula, x, decreasing=FALSE, na.last=TRUE) {
@@ -388,3 +405,4 @@ setMethod("xtabs", signature(data = "Vector"),
     data <- as.env(data, environment(formula), tform=decode)
     callGeneric()
 })
+
