@@ -50,15 +50,38 @@ void get_order_of_int_array(
 	int out_shift
 );
 
+int sort_ints(
+	int *base,
+	int base_len,
+	const int *x,
+	int desc,
+	int use_radix,
+	unsigned short int *rxbuf1,
+	int *rxbuf2
+);
+
 void get_order_of_int_pairs(
 	const int *a,
 	const int *b,
 	int nelt,
-	int desc,
+	int a_desc,
+	int b_desc,
 	int *out,
 	int out_shift
 );
 
+int sort_int_pairs(
+	int *base,
+	int base_len,
+	const int *a,
+	const int *b,
+	int a_desc,
+	int b_desc,
+	int use_radix,
+	unsigned short int *rxbuf1,
+	int *rxbuf2
+);
+
 void get_matches_of_ordered_int_pairs(
 	const int *a1,
 	const int *b1,
@@ -79,11 +102,30 @@ void get_order_of_int_quads(
 	const int *c,
 	const int *d,
 	int nelt,
-	int desc,
+	int a_desc,
+	int b_desc,
+	int c_desc,
+	int d_desc,
 	int *out,
 	int out_shift
 );
 
+int sort_int_quads(
+	int *base,
+	int base_len,
+	const int *a,
+	const int *b,
+	const int *c,
+	const int *d,
+	int a_desc,
+	int b_desc,
+	int c_desc,
+	int d_desc,
+	int use_radix,
+	unsigned short int *rxbuf1,
+	int *rxbuf2
+);
+
 void get_matches_of_ordered_int_quads(
 	const int *a1,
 	const int *b1,
@@ -379,12 +421,21 @@ int vector_memcmp(
 	int nelt
 );
 
-void vector_memcpy(
-	SEXP out,
-	int out_offset,
-	SEXP in,
-	int in_offset,
-	int nelt
+int copy_vector_block(
+	SEXP dest,
+	int dest_offset,
+	SEXP src,
+	int src_offset,
+	int block_width
+);
+
+int copy_vector_ranges(
+	SEXP dest,
+	int dest_offset,
+	SEXP src,
+	const int *start,
+	const int *width,
+	int nranges
 );
 
 SEXP list_as_data_frame(
@@ -424,11 +475,11 @@ SEXP find_interv_and_start_from_width(
  */
 
 SEXP new_Hits(
-	int *q_hits,
-	const int *s_hits,
+	int *from,
+	const int *to,
 	int nhit,
-	int q_len,
-	int s_len,
+	int nLnode,
+	int nRnode,
 	int already_sorted
 );
 
@@ -439,51 +490,51 @@ int get_select_mode(SEXP select);
  * (see Rle_class.c)
  */
 
-SEXP logical_Rle_constructor(
+SEXP construct_logical_Rle(
 	const int *values,
 	int nvalues,
 	const int *lengths,
 	int buflength
 );
 
-SEXP integer_Rle_constructor(
+SEXP construct_integer_Rle(
 	const int *values,
 	int nvalues,
 	const int *lengths,
 	int buflength
 );
 
-SEXP numeric_Rle_constructor(
+SEXP construct_numeric_Rle(
 	const double *values,
 	int nvalues,
 	const int *lengths,
 	int buflength
 );
 
-SEXP complex_Rle_constructor(
+SEXP construct_complex_Rle(
 	const Rcomplex *values,
 	int nvalues,
 	const int *lengths,
 	int buflength
 );
 
-SEXP character_Rle_constructor(
+SEXP construct_character_Rle(
 	SEXP values,
 	const int *lengths,
 	int buflength
 );
 
-SEXP raw_Rle_constructor(
+SEXP construct_raw_Rle(
 	const Rbyte *values,
 	int nvalues,
 	const int *lengths,
 	int buflength
 );
 
-SEXP seqselect_Rle(SEXP x,
-	const int *start,
-	const int *width,
-	int length
+SEXP construct_Rle(
+	SEXP values,
+	const int *lengths,
+	int buflength
 );
 
 /*
