@@ -65,9 +65,19 @@ DEFINE_NOVALUE_CCALLABLE_STUB(get_order_of_int_array,
 	(           x,     nelt,     desc,      out,     out_shift)
 )
 
+DEFINE_CCALLABLE_STUB(int, sort_ints,
+	(int *base, int base_len, const int *x, int desc, int use_radix, unsigned short int *rxbuf1, int *rxbuf2),
+	(     base,     base_len,            x,     desc,     use_radix,                     rxbuf1,      rxbuf2)
+)
+
 DEFINE_NOVALUE_CCALLABLE_STUB(get_order_of_int_pairs,
-	(const int *a, const int *b, int nelt, int desc, int *out, int out_shift),
-	(           a,            b,     nelt,     desc,      out,     out_shift)
+	(const int *a, const int *b, int nelt, int a_desc, int b_desc, int *out, int out_shift),
+	(           a,            b,     nelt,     a_desc,     b_desc,      out,     out_shift)
+)
+
+DEFINE_CCALLABLE_STUB(int, sort_int_pairs,
+	(int *base, int base_len, const int *a, const int *b, int a_desc, int b_desc, int use_radix, unsigned short int *rxbuf1, int *rxbuf2),
+	(     base,     base_len,            a,            b,     a_desc,     b_desc,     use_radix,                     rxbuf1,      rxbuf2)
 )
 
 DEFINE_NOVALUE_CCALLABLE_STUB(get_matches_of_ordered_int_pairs,
@@ -76,8 +86,13 @@ DEFINE_NOVALUE_CCALLABLE_STUB(get_matches_of_ordered_int_pairs,
 )
 
 DEFINE_NOVALUE_CCALLABLE_STUB(get_order_of_int_quads,
-	(const int *a, const int *b, const int *c, const int *d, int nelt, int desc, int *out, int out_shift),
-	(           a,            b,            c,            d,     nelt,     desc,      out,     out_shift)
+	(const int *a, const int *b, const int *c, const int *d, int nelt, int a_desc, int b_desc, int c_desc, int d_desc, int *out, int out_shift),
+	(           a,            b,            c,            d,     nelt,     a_desc,     b_desc,     c_desc,     d_desc,      out,     out_shift)
+)
+
+DEFINE_CCALLABLE_STUB(int, sort_int_quads,
+	(int *base, int base_len, const int *a, const int *b, const int *c, const int *d, int a_desc, int b_desc, int c_desc, int d_desc, int use_radix, unsigned short int *rxbuf1, int *rxbuf2),
+	(     base,     base_len,            a,            b,            c,            d,     a_desc,     b_desc,     c_desc,     d_desc,     use_radix,                     rxbuf1,      rxbuf2)
 )
 
 DEFINE_NOVALUE_CCALLABLE_STUB(get_matches_of_ordered_int_quads,
@@ -396,9 +411,14 @@ DEFINE_CCALLABLE_STUB(int, vector_memcmp,
 	(     x1,     x1_offset,      x2,     x2_offset,     nelt)
 )
 
-DEFINE_NOVALUE_CCALLABLE_STUB(vector_memcpy,
-	(SEXP out, int out_offset, SEXP in, int in_offset, int nelt),
-	(     out,     out_offset,      in,     in_offset,     nelt)
+DEFINE_CCALLABLE_STUB(int, copy_vector_block,
+	(SEXP dest, int dest_offset, SEXP src, int src_offset, int block_width),
+	(     dest,     dest_offset,      src,     src_offset,     block_width)
+)
+
+DEFINE_CCALLABLE_STUB(int, copy_vector_ranges,
+	(SEXP dest, int dest_offset, SEXP src, const int *start, const int *width, int nranges),
+	(     dest,     dest_offset,      src,            start,            width,     nranges)
 )
 
 DEFINE_CCALLABLE_STUB(SEXP, list_as_data_frame,
@@ -430,8 +450,8 @@ DEFINE_CCALLABLE_STUB(SEXP, find_interv_and_start_from_width,
  */
 
 DEFINE_CCALLABLE_STUB(SEXP, new_Hits,
-	(int *q_hits, const int *s_hits, int nhit, int q_len, int s_len, int already_sorted),
-	(     q_hits,            s_hits,     nhit,     q_len,     s_len,     already_sorted)
+	(int *from, const int *to, int nhit, int nLnode, int nRnode, int already_sorted),
+	(     from,            to,     nhit,     nLnode,     nRnode,     already_sorted)
 )
 
 DEFINE_CCALLABLE_STUB(int, get_select_mode,
@@ -443,39 +463,39 @@ DEFINE_CCALLABLE_STUB(int, get_select_mode,
  * Stubs for callables defined in Rle_class.c
  */
 
-DEFINE_CCALLABLE_STUB(SEXP, logical_Rle_constructor,
+DEFINE_CCALLABLE_STUB(SEXP, construct_logical_Rle,
 	(const int *values, int nvalues, const int *lengths, int buflength),
 	(           values,     nvalues,            lengths,     buflength)
 )
 
-DEFINE_CCALLABLE_STUB(SEXP, integer_Rle_constructor,
+DEFINE_CCALLABLE_STUB(SEXP, construct_integer_Rle,
 	(const int *values, int nvalues, const int *lengths, int buflength),
 	(           values,     nvalues,            lengths,     buflength)
 )
 
-DEFINE_CCALLABLE_STUB(SEXP, numeric_Rle_constructor,
+DEFINE_CCALLABLE_STUB(SEXP, construct_numeric_Rle,
 	(const double *values, int nvalues, const int *lengths, int buflength),
 	(              values,     nvalues,            lengths,     buflength)
 )
 
-DEFINE_CCALLABLE_STUB(SEXP, complex_Rle_constructor,
+DEFINE_CCALLABLE_STUB(SEXP, construct_complex_Rle,
 	(const Rcomplex *values, int nvalues, const int *lengths, int buflength),
 	(                values,     nvalues,            lengths,     buflength)
 )
 
-DEFINE_CCALLABLE_STUB(SEXP, character_Rle_constructor,
+DEFINE_CCALLABLE_STUB(SEXP, construct_character_Rle,
 	(SEXP values, const int *lengths, int buflength),
 	(     values,            lengths,     buflength)
 )
 
-DEFINE_CCALLABLE_STUB(SEXP, raw_Rle_constructor,
+DEFINE_CCALLABLE_STUB(SEXP, construct_raw_Rle,
 	(const Rbyte *values, int nvalues, const int *lengths, int buflength),
 	(             values,     nvalues,            lengths,     buflength)
 )
 
-DEFINE_CCALLABLE_STUB(SEXP, seqselect_Rle,
-	(SEXP x, const int *start, const int *width, int length),
-	(     x,            start,            width,     length)
+DEFINE_CCALLABLE_STUB(SEXP, construct_Rle,
+	(SEXP values, const int *lengths, int buflength),
+	(     values,            lengths,     buflength)
 )
 
 /*
