@@ -9,7 +9,6 @@
 static const R_CallMethodDef callMethods[] = {
 
 /* AEbufs.c */
-	CALLMETHOD_DEF(debug_AEbufs, 0),
 	CALLMETHOD_DEF(AEbufs_use_malloc, 1),
 	CALLMETHOD_DEF(AEbufs_free, 0),
 
@@ -17,9 +16,8 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(anyMissing, 1),
 
 /* vector_utils.c */
+	CALLMETHOD_DEF(vectorORfactor_extract_ranges, 3),
 	CALLMETHOD_DEF(sapply_NROW, 1),
-	CALLMETHOD_DEF(vector_subsetByRanges, 3),
-	CALLMETHOD_DEF(vector_seqselect, 3),
 
 /* logical_utils.c */
 	CALLMETHOD_DEF(logical_as_compact_bitvector, 1),
@@ -34,15 +32,16 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(Integer_sum_non_neg_vals, 1),
 	CALLMETHOD_DEF(Integer_diff_with_0, 1),
 	CALLMETHOD_DEF(Integer_diff_with_last, 2),
-	CALLMETHOD_DEF(Integer_order, 2),
-	CALLMETHOD_DEF(Integer_compare2, 4),
+	CALLMETHOD_DEF(Integer_order, 3),
+	CALLMETHOD_DEF(Integer_pcompare2, 4),
 	CALLMETHOD_DEF(Integer_sorted2, 4),
-	CALLMETHOD_DEF(Integer_order2, 3),
+	CALLMETHOD_DEF(Integer_order2, 4),
 	CALLMETHOD_DEF(Integer_match2_quick, 5),
 	CALLMETHOD_DEF(Integer_selfmatch2_quick, 2),
 	CALLMETHOD_DEF(Integer_match2_hash, 5),
 	CALLMETHOD_DEF(Integer_selfmatch2_hash, 2),
-	CALLMETHOD_DEF(Integer_order4, 5),
+	CALLMETHOD_DEF(Integer_sorted4, 6),
+	CALLMETHOD_DEF(Integer_order4, 6),
 	CALLMETHOD_DEF(Integer_match4_quick, 9),
 	CALLMETHOD_DEF(Integer_selfmatch4_quick, 4),
 	CALLMETHOD_DEF(Integer_match4_hash, 9),
@@ -64,11 +63,8 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(top_prenv, 2),
 	CALLMETHOD_DEF(top_prenv_dots, 1),
 
-/* subsetting_internals.c */
-	CALLMETHOD_DEF(vector_extract_window, 3),
-
 /* Hits_class.c */
-	CALLMETHOD_DEF(Hits_new, 5),
+	CALLMETHOD_DEF(Hits_new, 6),
 	CALLMETHOD_DEF(select_hits, 4),
 	CALLMETHOD_DEF(make_all_group_inner_hits, 2),
 
@@ -76,10 +72,11 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(Rle_constructor, 4),
 	CALLMETHOD_DEF(Rle_start, 1),
 	CALLMETHOD_DEF(Rle_end, 1),
+	CALLMETHOD_DEF(ranges_to_runs_mapper, 4),
+	CALLMETHOD_DEF(Rle_extract_range, 3),
+	CALLMETHOD_DEF(Rle_extract_ranges, 5),
 	CALLMETHOD_DEF(Rle_getStartEndRunAndOffset, 3),
 	CALLMETHOD_DEF(Rle_window_aslist, 5),
-	CALLMETHOD_DEF(Rle_window, 6),
-	CALLMETHOD_DEF(Rle_seqselect, 3),
 
 /* Rle_utils.c */
 	CALLMETHOD_DEF(Rle_runsum, 3),
@@ -101,9 +98,11 @@ void R_init_S4Vectors(DllInfo *info)
 	REGISTER_CCALLABLE(_safe_int_mult);
 
 /* sort_utils.c */
-	REGISTER_CCALLABLE(_sort_int_array);
+	REGISTER_CCALLABLE(_sort_ints);
 	REGISTER_CCALLABLE(_get_order_of_int_array);
+	REGISTER_CCALLABLE(_sort_int_array);
 	REGISTER_CCALLABLE(_get_order_of_int_pairs);
+	REGISTER_CCALLABLE(_sort_int_pairs);
 	REGISTER_CCALLABLE(_get_matches_of_ordered_int_pairs);
 	REGISTER_CCALLABLE(_get_order_of_int_quads);
 	REGISTER_CCALLABLE(_get_matches_of_ordered_int_quads);
@@ -174,7 +173,8 @@ void R_init_S4Vectors(DllInfo *info)
 
 /* vector_utils.c */
 	REGISTER_CCALLABLE(_vector_memcmp);
-	REGISTER_CCALLABLE(_vector_memcpy);
+	REGISTER_CCALLABLE(_copy_vector_block);
+	REGISTER_CCALLABLE(_copy_vector_ranges);
 	REGISTER_CCALLABLE(_list_as_data_frame);
 
 /* int_utils.c */
@@ -187,13 +187,13 @@ void R_init_S4Vectors(DllInfo *info)
 	REGISTER_CCALLABLE(_get_select_mode);
 
 /* Rle_class.c */
-	REGISTER_CCALLABLE(_logical_Rle_constructor);
-	REGISTER_CCALLABLE(_integer_Rle_constructor);
-	REGISTER_CCALLABLE(_numeric_Rle_constructor);
-	REGISTER_CCALLABLE(_complex_Rle_constructor);
-	REGISTER_CCALLABLE(_character_Rle_constructor);
-	REGISTER_CCALLABLE(_raw_Rle_constructor);
-	REGISTER_CCALLABLE(_seqselect_Rle);
+	REGISTER_CCALLABLE(_construct_logical_Rle);
+	REGISTER_CCALLABLE(_construct_integer_Rle);
+	REGISTER_CCALLABLE(_construct_numeric_Rle);
+	REGISTER_CCALLABLE(_construct_complex_Rle);
+	REGISTER_CCALLABLE(_construct_character_Rle);
+	REGISTER_CCALLABLE(_construct_raw_Rle);
+	REGISTER_CCALLABLE(_construct_Rle);
 
 /* List_class.c */
 	REGISTER_CCALLABLE(_get_List_elementType);
