@@ -1,8 +1,6 @@
 #include "../inst/include/S4Vectors_defines.h"
 #include <string.h>
 
-#define DEBUG_S4VECTORS 1
-
 #define INIT_STATIC_SYMBOL(NAME) \
 { \
 	if (NAME ## _symbol == NULL) \
@@ -43,7 +41,17 @@ void _get_order_of_int_array(
 	int out_shift
 );
 
-void _compare_int_pairs(
+int _sort_ints(
+	int *base,
+	int base_len,
+	const int *x,
+	int desc,
+	int use_radix,
+	unsigned short int *rxbuf1,
+	int *rxbuf2
+);
+
+void _pcompare_int_pairs(
 	const int *a1,
 	const int *b1,
 	int nelt1,
@@ -67,11 +75,24 @@ void _get_order_of_int_pairs(
 	const int *a,
 	const int *b,
 	int nelt,
-	int desc,
+	int a_desc,
+	int b_desc,
 	int *out,
 	int out_shift
 );
 
+int _sort_int_pairs(
+	int *base,
+	int base_len,
+	const int *a,
+	const int *b,
+	int a_desc,
+	int b_desc,
+	int use_radix,
+	unsigned short int *rxbuf1,
+	int *rxbuf2
+);
+
 void _get_matches_of_ordered_int_pairs(
 	const int *a1,
 	const int *b1,
@@ -86,17 +107,46 @@ void _get_matches_of_ordered_int_pairs(
 	int out_shift
 );
 
-void _get_order_of_int_quads(
+int _int_quads_are_sorted(
 	const int *a,
 	const int *b,
 	const int *c,
 	const int *d,
 	int nelt,
 	int desc,
+	int strict
+);
+
+void _get_order_of_int_quads(
+	const int *a,
+	const int *b,
+	const int *c,
+	const int *d,
+	int nelt,
+	int a_desc,
+	int b_desc,
+	int c_desc,
+	int d_desc,
 	int *out,
 	int out_shift
 );
 
+int _sort_int_quads(
+	int *base,
+	int base_len,
+	const int *a,
+	const int *b,
+	const int *c,
+	const int *d,
+	int a_desc,
+	int b_desc,
+	int c_desc,
+	int d_desc,
+	int use_radix,
+	unsigned short int *rxbuf1,
+	int *rxbuf2
+);
+
 void _get_matches_of_ordered_int_quads(
 	const int *a1,
 	const int *b1,
@@ -134,8 +184,6 @@ void _set_hbucket_val(
 
 /* AEbufs.c */
 
-SEXP debug_AEbufs();
-
 SEXP AEbufs_use_malloc(SEXP x);
 
 int _get_new_buflength(int buflength);
@@ -397,28 +445,38 @@ int _vector_memcmp(
 	int nelt
 );
 
-void _vector_memcpy(
-	SEXP out,
-	int out_offset,
-	SEXP in,
-	int in_offset,
-	int nelt
+int _copy_vector_block(
+	SEXP dest,
+	int dest_offset,
+	SEXP src,
+	int src_offset,
+	int block_width
+);
+
+int _copy_vector_ranges(
+	SEXP dest,
+	int dest_offset,
+	SEXP src,
+	const int *start,
+	const int *width,
+	int nranges
+);
+
+SEXP _subset_vectorORfactor_by_ranges(
+	SEXP x,
+	const int *start,
+	const int *width,
+	int nranges
+);
+
+SEXP vectorORfactor_extract_ranges(
+	SEXP x,
+	SEXP start,
+	SEXP width
 );
 
 SEXP sapply_NROW(SEXP x);
 
-SEXP vector_subsetByRanges(
-	SEXP x,
-	SEXP start,
-	SEXP width
-);
-
-SEXP vector_seqselect(
-	SEXP x,
-	SEXP start,
-	SEXP width
-);
-
 SEXP _list_as_data_frame(
 	SEXP x,
 	int nrow
@@ -458,7 +516,8 @@ SEXP Integer_diff_with_last(SEXP x, SEXP last);
 
 SEXP Integer_order(
 	SEXP x,
-	SEXP decreasing
+	SEXP decreasing,
+	SEXP use_radix
 );
 
 int _check_integer_pairs(
@@ -470,7 +529,7 @@ int _check_integer_pairs(
 	const char *b_argname
 );
 
-SEXP Integer_compare2(
+SEXP Integer_pcompare2(
 	SEXP a1,
 	SEXP b1,
 	SEXP a2,
@@ -487,7 +546,8 @@ SEXP Integer_sorted2(
 SEXP Integer_order2(
 	SEXP a,
 	SEXP b,
-	SEXP decreasing
+	SEXP decreasing,
+	SEXP use_radix
 );
 
 SEXP Integer_match2_quick(
@@ -531,12 +591,22 @@ int _check_integer_quads(
 	const char *d_argname
 );
 
+SEXP Integer_sorted4(
+	SEXP a,
+	SEXP b,
+	SEXP c,
+	SEXP d,
+	SEXP decreasing,
+	SEXP strictly
+);
+
 SEXP Integer_order4(
 	SEXP a,
 	SEXP b,
 	SEXP c,
 	SEXP d,
-	SEXP decreasing
+	SEXP decreasing,
+	SEXP use_radix
 );
 
 SEXP Integer_match4_quick(
@@ -636,36 +706,32 @@ SEXP top_prenv(SEXP nm, SEXP env);
 SEXP top_prenv_dots(SEXP env);
 
 
-/* subsetting_internals.c */
-
-SEXP vector_extract_window(SEXP x, SEXP start, SEXP end);
-
-
 /* Hits_class.c */
 
 SEXP _new_Hits(
-	int *q_hits,
-	const int *s_hits,
+	int *from,
+	const int *to,
 	int nhit,
-	int q_len,
-	int s_len,
+	int nLnode,
+	int nRnode,
 	int already_sorted
 );
 
 SEXP Hits_new(
-	SEXP q_hits,
-	SEXP s_hits,
-	SEXP q_len,
-	SEXP s_len,
+	SEXP Class,
+	SEXP from,
+	SEXP to,
+	SEXP nLnode,
+	SEXP nRnode,
 	SEXP revmap_envir
 );
 
 int _get_select_mode(SEXP select);
 
 SEXP select_hits(
-	SEXP q_hits,
-	SEXP s_hits,
-	SEXP q_len,
+	SEXP from,
+	SEXP to,
+	SEXP nLnode,
 	SEXP select
 );
 
@@ -677,47 +743,53 @@ SEXP make_all_group_inner_hits(
 
 /* Rle_class.c */
 
-SEXP _logical_Rle_constructor(
+SEXP _construct_logical_Rle(
 	const int *values,
 	int nvalues,
 	const int *lengths,
 	int buflength
 );
 
-SEXP _integer_Rle_constructor(
+SEXP _construct_integer_Rle(
 	const int *values,
 	int nvalues,
 	const int *lengths,
 	int buflength
 );
 
-SEXP _numeric_Rle_constructor(
+SEXP _construct_numeric_Rle(
 	const double *values,
 	int nvalues,
 	const int *lengths,
 	int buflength
 );
 
-SEXP _complex_Rle_constructor(
+SEXP _construct_complex_Rle(
 	const Rcomplex *values,
 	int nvalues,
 	const int *lengths,
 	int buflength
 );
 
-SEXP _character_Rle_constructor(
+SEXP _construct_character_Rle(
 	SEXP values,
 	const int *lengths,
 	int buflength
 );
 
-SEXP _raw_Rle_constructor(
+SEXP _construct_raw_Rle(
 	const Rbyte *values,
 	int nvalues,
 	const int *lengths,
 	int buflength
 );
 
+SEXP _construct_Rle(
+	SEXP values,
+	const int *lengths,
+	int buflength
+);
+
 SEXP Rle_constructor(
 	SEXP values,
 	SEXP lengths,
@@ -729,6 +801,36 @@ SEXP Rle_start(SEXP x);
 
 SEXP Rle_end(SEXP x);
 
+SEXP ranges_to_runs_mapper(
+	SEXP run_lengths,
+	SEXP start,
+	SEXP width,
+	SEXP method
+);
+
+SEXP Rle_extract_range(
+	SEXP x,
+	SEXP start,
+	SEXP end
+);
+
+SEXP _subset_Rle_by_ranges(
+	SEXP x,
+	const int *start,
+	const int *width,
+	int nranges,
+	int method,
+	int as_list
+);
+
+SEXP Rle_extract_ranges(
+	SEXP x,
+	SEXP start,
+	SEXP width,
+	SEXP method,
+	SEXP as_list
+);
+
 SEXP Rle_getStartEndRunAndOffset(
 	SEXP x,
 	SEXP start,
@@ -743,28 +845,6 @@ SEXP Rle_window_aslist(
 	SEXP offsetEnd
 );
 
-SEXP Rle_window(
-	SEXP x,
-	SEXP runStart,
-	SEXP runEnd,
-	SEXP offsetStart,
-	SEXP offsetEnd,
-	SEXP ans
-);
-
-SEXP _seqselect_Rle(
-	SEXP x,
-	const int *start,
-	const int *width,
-	int length
-);
-
-SEXP Rle_seqselect(
-	SEXP x,
-	SEXP start,
-	SEXP width
-);
-
 
 /* Rle_utils.c */
 
